# ugrp engine - User, Groups, Roles & Permissions

## Setup

``` bash
rails plugin new ugrp --mountable -d sqlite3 -T --dummy-path=spec/dummy --skip-bundle
```

Add the following to the gemspec file:

``` ruby
  s.rdoc_options = ["--line-numbers", "--inline-source", "--title", "ugrp", "--main", "README.rdoc"]
  s.extra_rdoc_files = ["CHANGELOG", "README.rdoc", "lib/tasks/ugrp_tasks.rake", "lib/ugrp.rb"]
```

AND

``` ruby
  s.add_development_dependency 'rcov', '~> 0.9.9'
  s.add_development_dependency 'rspec-rails', '~> 2.8.1'
  s.add_development_dependency 'cucumber-rails', '~> 1.3.0'
  s.add_development_dependency 'shoulda-matchers', '~> 1.2.0'
  s.add_development_dependency 'factory_bot_rails', '~> 1.7.0'
  s.add_development_dependency 'database_cleaner', '~> 0.7.1'
  s.add_development_dependency 'ci_reporter', '~> 1.7.0'
```

### Get basic gem generation working

``` bash
rake build
```

### RSpec

``` bash
bundle exec rails generate rspec:install
```

#### Tweaks for 'spec_helper.rb'

``` ruby
# This file is copied to spec/ when you run 'rails generate rspec:install'
ENV["RAILS_ENV"] ||= 'test'
require File.expand_path("../../config/environment.rb", __FILE__)
require File.expand_path("../../spec/dummy/config/environment.rb", __FILE__)
#require File.expand_path("../../test/dummy/config/environments/#{::Rails.env}.rb", __FILE__)

require 'rspec/rails'
require 'rspec/autorun'

require 'factory_bot_rails'
require 'database_cleaner'
require 'shoulda/matchers/integrations/rspec'

ENGINE_RAILS_ROOT=File.join(File.dirname(__FILE__), '../')

# Requires supporting ruby files with custom matchers and macros, etc,
# in spec/support/ and its subdirectories.
# Dir[Rails.root.join("spec/support/**/*.rb")].each {|f| require f}
Dir[File.join(ENGINE_RAILS_ROOT, 'spec/support/**/*.rb')].each {|f| require f }

RSpec.configure do |config|
  # ## Mock Framework
  #
  # If you prefer to use mocha, flexmock or RR, uncomment the appropriate line:
  #
  # config.mock_with :mocha
  # config.mock_with :flexmock
  # config.mock_with :rr
  config.mock_with :rspec

  # Remove this line if you're not using ActiveRecord or ActiveRecord fixtures
  config.fixture_path = "#{::Rails.root}/spec/fixtures"

  # If you're not using ActiveRecord, or you'd prefer not to run each of your
  # examples within a transaction, remove the following line or assign false
  # instead of true.
  config.use_transactional_fixtures = false

  # Use DatabaseCleaner to manage transactions
  config.before(:suite) do
    DatabaseCleaner.strategy = :transaction
    DatabaseCleaner.clean_with(:truncation)
  end
  config.before(:each) do
    DatabaseCleaner.start
  end
  config.after(:each) do
    DatabaseCleaner.clean
  end

  # If true, the base class of anonymous controllers will be inferred
  # automatically. This will be the default behavior in future versions of
  # rspec-rails.
  config.infer_base_class_for_anonymous_controllers = false

  # Include Engine routes (needed for Controller specs)
  config.include Ugrp::Engine.routes.url_helpers
end

Test RSpec will run:
bundle exec rspec
```

### Cucumber

bundle exec rails generate cucumber:install --rspec --capybara

#### Tweaks for 'env.rb'

``` ruby
ENV["RAILS_ENV"] ||= "test"
ENV["RAILS_ROOT"] ||= File.dirname(__FILE__) + "../../../spec/dummy"
require File.expand_path("../../../spec/dummy/config/environment.rb",  __FILE__)

require 'cucumber/rails'
```

--- AND ---

``` ruby
# include Engine routes in Cucumber world
module EngineRoutesHelper
  include AccountsManagement::Engine.routes.url_helpers
end
World(EngineRoutesHelper)
```

Test Cucumber will run:

``` bash
bundle exec cucumber
```

== Model definitions:

=== UGRP (Users, Groups, Roles & Permissions)

``` bash
bundle exec rails generate scaffold Role name:string descriptive_name:string is_active:boolean
bundle exec rails generate scaffold Group name:string descriptive_name:string is_active:boolean
bundle exec rails generate scaffold Permission name:string descriptive_name:string is_active:boolean
bundle exec rails generate scaffold UserPreference user:references key:string value:string

bundle exec rails destroy scaffold Role
bundle exec rails destroy scaffold Group
bundle exec rails destroy scaffold Permission

bundle exec rails generate controller Dashboard index
```

== CAPTCHA/reCAPTCHA

[Google reCAPTCHA](https://www.google.com/recaptcha) / [Google reCAPTCHA admin](https://www.google.com/recaptcha/admin)

## Development with docker-compose

``` bash
(docker-compose build --build-arg BUILD_ENV=local && docker-compose --project-name microservices up && docker-compose --project-name microservices down)
```

## Deployment to kubernetes

### Create kubernetes namespace

``` bash
(kubectl get namespace "microservices" || kubectl create namespace "microservices")
kubectl label namespace microservices istio-injection=enabled
```

### Create regcred

``` bash
kubectl --namespace="microservices" delete secret regcred
```

``` bash
kubectl create secret docker-registry regcred \
  --namespace="microservices" \
  --docker-server=docker-registry.localnet.htcl.eu \
  --docker-email=name@domain.com \
  --docker-username=everard \
  --docker-password=3...l
```

``` bash
kubectl create secret docker-registry regcred \
  --namespace="microservices" \
  --docker-server=artifactory.localnet.htcl.eu:5001 \
  --docker-email=name@domain.com \
  --docker-username=everard \
  --docker-password=3...L
```

### Update/Fix nginx-ingress-controller configuration

#### RKE

``` bash
kubectl apply -f kubernetes/rke-nginx-configuration.yaml
```

#### RKE2

``` bash
kubectl apply -f kubernetes/rke2-nginx-configuration.yaml
```

### Image creation/deployment

``` bash
time docker-compose build --build-arg BUILD_ENV=local --no-cache --pull
  OR
time docker-compose build --build-arg BUILD_ENV=local --build-arg TARGET_ENVIRONMENT=':localhost'
  OR
time docker-compose build --build-arg BUILD_ENV=local --build-arg TARGET_ENVIRONMENT=':k8s-localhost'
  OR
time docker-compose build --build-arg BUILD_ENV=local --build-arg RAILS_ENV=production
```

  OR

``` bash
time docker build --build-arg BUILD_ENV=local --no-cache --pull --tag ugrp .
  OR
time docker build --build-arg BUILD_ENV=local ---build-arg TARGET_ENVIRONMENT=':localhost' -tag ugrp .
  OR
time docker build --build-arg BUILD_ENV=local --build-arg TARGET_ENVIRONMENT=':k8s-localhost' --tag ugrp .
  OR
time docker build --build-arg BUILD_ENV=local --build-arg RAILS_ENV=production --tag ugrp .
```

``` bash
# image="docker-registry.localnet.htcl.eu/ugrp:latest"
image="docker-registry.localnet.htcl.eu/ugrp:$(date +%Y%m%d%H%M%S)_$(git --no-pager log -1 --pretty=%h)" ; \
latest_image="docker-registry.localnet.htcl.eu/ugrp:latest" && \
docker tag ugrp $image && docker tag ugrp $latest_image && \
docker push $image && docker push $latest_image && \
docker rmi $image && docker rmi $latest_image
```

### Application deployment

``` bash
export DATABASE_NAME=ugrp_engine_db
# export DATABASE_HOSTNAME=postgresql.default.svc.cluster.local
export DATABASE_HOSTNAME=postgresql.default
export DATABASE_USER=postgres
export DATABASE_PASSWORD=$(kubectl get secret --namespace default postgresql -o jsonpath="{.data.postgres-password}" | base64 -d)
```

``` bash
./kubernetes/deploy --namespace "microservices" --image $image --ingress ugrp.localnet.htcl.eu,ugrp.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --include 'ugrp|persistentvolumeclaim' --exclude 'postgres|pgadmin' --env-file .env.k8s-localhost --verbose

./kubernetes/deploy --namespace "microservices" --image $image --ingress ugrp.localnet.htcl.eu,ugrp.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --include 'ugrp|persistentvolumeclaim' --env-file .env.k8s-localhost --verbose

./kubernetes/deploy --namespace "microservices" --image $image --ingress ugrp.localnet.htcl.eu,ugrp.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --verbose

./kubernetes/deploy --namespace "microservices" --image $image --ingress ugrp.localnet.htcl.eu,ugrp.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --k8s-server k8s-localhost-centos8.localnet.htcl.eu --k8s-port 6443 --verbose

./kubernetes/deploy --namespace "microservices"  --image $image --ingress ugrp.localnet.htcl.eu,ugrp.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --verbose --redeploy --dry-run

./kubernetes/deploy --namespace "microservices" --image $image --ingress ugrp.localnet.htcl.eu,ugrp.apps.localnet.htcl.eu --activate-tls-on-ingress --replicas 1 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG --rails-env production --verbose
```

Spare parts

``` bash
 --redeploy
 --dry-run

 --replicas POD_REPLICAS
 --gitlab-ci-project-path-slug CI_PROJECT_PATH_SLUG
 --gitlab-ci-environment-slug CI_ENVIRONMENT_SLUG

 --pvc-storageclassname "default"
 --pvc-storageclassname "local-path"
 --pvc-storageclassname "longhorn"
```

### Check deployment

``` bash
helm list --all --namespace "microservices"
helm list --all --all-namespaces
```

---

## git-svn notes

Import project from subversion repository:

``` bash
git svn clone --trunk=trunk/ugrp --tags=tags/ugrp --branches=branches/ugrp --authors-file=/home/everard/.git-authors https://dev.htcl.eu/project/repo/website/rails_engines ugrp
```

New branch linked to subversion repository:

``` bash
git svn branch -m "Create feature branch" <feature_branch>
git checkout --track -b <feature_branch> <feature_branch>
```

Integration:

``` bash
git merge <feature_branch>
```

- OR -

``` bash
git merge --squash <feature_branch>
```

Get latest changes from subversion repository:

``` bash
git svn rebase
```

Commit to subversion repository:

``` bash
git svn rebase && git svn dcommit
```

Synchronising branches:

``` bash
git stash save && git checkout master && git svn rebase && git checkout wip && git rebase master && git checkout master && git merge wip && git svn dcommit && git checkout wip && git rebase master && git stash pop
```

OR

``` bash
git stash save && git checkout master && git svn rebase && git checkout wip && git rebase master && git stash pop
```

Integrate 'wip' into 'development':

``` bash
git stash save && git checkout wip && git rebase development && git checkout development && git merge wip && git checkout wip && git rebase development && git stash pop
```

Integrate 'development' into 'master':

``` bash
git stash save && git checkout master && git svn rebase && git checkout development && git rebase master && git checkout master && git merge development && git svn dcommit && git checkout development && git rebase master && git stash pop
```

Re-base 'wip' from 'development':

``` bash
git stash save && git checkout wip && git rebase development && git stash pop
```

Re-base 'development' from 'master':

``` bash
git stash save && git checkout master && git svn rebase && git checkout development && git rebase master && git stash pop
```

Sync 'master' to subversion

``` bash
git stash save && git checkout master && git svn dcommit && git stash pop
```

Sync to all shared repositories:

``` bash
git gc && for branch in master development ; do [[ `git rev-parse --verify --quiet $branch` ]] && (git stash save 'repo-sync' && git pull --rebase gitlab $branch && ([[ `git stash list | grep 'repo-sync'` ]] && git stash pop) ; (for repo in origin tfs gitlab ; do git push -u $repo $branch ; done)) ; done
```

--
This project rocks and is copyrighted.
