# frozen_string_literal: true

require 'cancancan'

module Ugrp
  class Ability
    include ::CanCan::Ability

    def initialize(user)
      # Define abilities for the passed in user here. For example:
      #
      #   user ||= User.new # guest user (not logged in)
      #   if user.admin?
      #     can :manage, :all
      #   else
      #     can :read, :all
      #   end
      #
      # The first argument to `can` is the action you are giving the user permission to do.
      # If you pass :manage it will apply to every action. Other common actions here are
      # :read, :create, :update and :destroy.
      #
      # The second argument is the resource the user can perform the action on. If you pass
      # :all it will apply to every resource. Otherwise pass a Ruby class of the resource.
      #
      # The third argument is an optional hash of conditions to further filter the objects.
      # For example, here the user can only update published articles.
      #
      #   can :update, Article, published: true
      #
      # See the wiki for details: https://github.com/ryanb/cancan/wiki/Defining-Abilities

      user ||= Ugrp::User.new # guest user (not logged in)

      can :manage, :all if user.has_role? :r_developer
      can :manage, :all if user.has_role? :r_super_user

      default_domain = Ugrp::Domain.default_domain
      roles = user.roles.where(domain: [default_domain, user.current_domain || nil])
      groups = user.groups.where(domain: [default_domain, user.current_domain || nil])

      return if roles.nil? && groups.nil?

      # Get applicable Ugrp::AclMapping data
      acl_mappings = Ugrp::AclMapping.where(group_or_role: roles) + Ugrp::AclMapping.where(group_or_role: groups)

      acl_mappings.each do |acl_mapping|
        acl_reference = acl_mapping.object_name
        acl_reference += ('_' + acl_mapping.field_name) unless acl_mapping.field_name.nil?

        can :create,  acl_reference.singularize.to_sym if acl_mapping.can_create
        can :read,    acl_reference.singularize.to_sym if acl_mapping.can_read
        can :update,  acl_reference.singularize.to_sym if acl_mapping.can_update
        can :delete,  acl_reference.singularize.to_sym if acl_mapping.can_delete
        can :destroy, acl_reference.singularize.to_sym if acl_mapping.can_delete
      end

      # Get applicable Ugrp::Permission data
    end
  end
end
