# frozen_string_literal: true

module Ugrp
  class User < ActiveRecord::Base
    belongs_to :user_account_owner, polymorphic: true, optional: true
    accepts_nested_attributes_for :user_account_owner

    # include Ugrp::TokenAuthenticatable

    # self.table_name = 'ugrp_users'

    # Include default devise modules. Others available are:
    # :token_authenticatable, :encryptable, :confirmable, :lockable, :timeoutable and :omniauthable
    #
    # NOTES:
    # :token_authenticatable is deprecated
    # :encryptable might be deprecated
    #
    if Gem.loaded_specs.key?('omniauth')
      devise :database_authenticatable, :recoverable, :rememberable, :trackable, :registerable, :confirmable, :validatable,
             :lockable, :timeoutable, :omniauthable
    else
      devise :database_authenticatable, :recoverable, :rememberable, :trackable, :registerable, :confirmable, :validatable,
             :lockable, :timeoutable
    end

    # Setup accessible (or protected) attributes for your model
    # attr_accessible :username, :display_name
    # attr_accessible :name, :email, :password, :password_confirmation, :remember_me, :confirmed_at
    # attr_accessible :login

    # Virtual attributes
    # ------------------
    # for authenticating by either username or email
    # This is in addition to a real persisted field like 'username'
    attr_accessor :login

    #
    # current domain - determined from the request string
    mattr_accessor :current_domain
    #
    # current user - determined from devise (before_filter :update_ugrp_user)
    mattr_accessor :current_user

    #----------------------------------------------------------------------------

    has_many :preferences, class_name: 'Ugrp::UserPreference', dependent: :destroy
    accepts_nested_attributes_for :preferences

    # has_and_belongs_to_many :domains
    # has_and_belongs_to_many :domains, through: :user_domains, class_name: 'Ugrp::Domain'
    has_many :user_domains, class_name: 'Ugrp::UserDomain', dependent: :destroy
    accepts_nested_attributes_for :user_domains
    has_many :domains, through: :user_domains, class_name: 'Ugrp::Domain'
    accepts_nested_attributes_for :domains

    # has_and_belongs_to_many :groups
    # has_and_belongs_to_many :groups, through: :user_groups, class_name: 'Ugrp::Group'
    has_many :user_groups, class_name: 'Ugrp::UserGroup', dependent: :destroy
    accepts_nested_attributes_for :user_groups
    has_many :groups, through: :user_groups, class_name: 'Ugrp::Group'
    accepts_nested_attributes_for :groups

    # has_and_belongs_to_many :roles
    # has_and_belongs_to_many :roles, through: :user_roles, class_name: 'Ugrp::Role'
    has_many :user_roles, class_name: 'Ugrp::UserRole', dependent: :destroy
    accepts_nested_attributes_for :user_roles
    has_many :roles, through: :user_roles, class_name: 'Ugrp::Role'
    accepts_nested_attributes_for :roles

    # has_and_belongs_to_many :permissions
    # has_and_belongs_to_many :permissions, through: :user_permissions, class_name: 'Ugrp::Permission'
    has_many :user_permissions, class_name: 'Ugrp::UserPermission', dependent: :destroy
    accepts_nested_attributes_for :user_permissions
    has_many :permissions, through: :user_permissions, class_name: 'Ugrp::Permission'
    accepts_nested_attributes_for :permissions

    #----------------------------------------------------------------------------

    # TODO: This needs is changed for rails 6.1, since there is no column called 'domains'
    # validates :username, uniqueness: { scope: :domains, message: 'already exists in the domain' }

    # TODO: These validations need to be completed
    validate :user_must_be_associated_to_a_domain
    validate :username_is_unique_in_domain

    def user_must_be_associated_to_a_domain
      # By default, the user account should be associated to the default domain
      # It should not cause a validation error
      # if domains.empty?
      #   errors.add(:username, 'has not been associated to a domain')
      # end
    end

    def username_is_unique_in_domain
      nil if domains.empty?

      # if
      #   errors.add(:username, 'already exists in the domain <>name>')
      # end
    end

    #----------------------------------------------------------------------------

    before_save :ensure_authentication_token

    #----------------------------------------------------------------------------

    scope :unconfirmed, -> { where(confirmed_at: nil) }
    scope :confirmed, -> { where.not(confirmed_at: nil) }
    scope :active, -> { where(is_active: true) }

    # Yes, I know these 2 are the same
    scope :active_and_confirmed, -> { active.confirmed }
    scope :confirmed_and_active, -> { confirmed.active }

    #------------------------------------------------------------------

    # Handlers for polymorphic references (using Global ID)

    # Use :to_global_id to populate forms
    def user_account_owner_gid
      user_account_owner&.to_global_id
    end

    # Set the attribute from a Global ID (handles the form submission)
    def user_account_owner_gid=(gid)
      self.user_account_owner = GlobalID::Locator.locate gid
    end

    #----------------------------------------------------------------------------

    def to_s
      best_display_name
    end

    def best_display_name
      display_name = user_account_owner.to_s unless user_account_owner.nil?

      display_name.presence || name.presence || username.presence || email
    end

    #----------------------------------------------------------------------------

    def current_role
      available_roles = []
      # Default roles
      available_roles.concat(roles.where(domain: Ugrp::Domain.default_domain))
      # Custom roles
      available_roles.concat(roles.where(domain: current_domain || nil))

      # FIXME: This makes assumptions about the order and precedence of avaialable_roles
      role = if avaialable_roles.empty?
               nil
             else
               avaialable_roles.size == 1 ? avaialable_roles.first : nil
             end

      logger.debug "current_role = '" + (role.nil? ? 'NONE' : role.name.to_s) + "'"
      role
    end

    def current_group
      avaialable_groups = []
      # Default groups
      avaialable_groups.concat(groups.where(domain: Ugrp::Domain.default_domain))
      # Custom groups
      avaialable_groups.concat(groups.where(domain: current_domain || nil))

      # FIXME: This makes assumptions about the order and precedence of avaialable_groups
      group = if avaialable_groups.empty?
                nil
              else
                avaialable_groups.size == 1 ? avaialable_groups.first : nil
              end

      logger.debug "current_group = '" + (group.nil? ? 'NONE' : group.name.to_s) + "'"
      group
    end

    def belongs_to_domain?(domain)
      belongs_to_domain_named?(domain.name)
    end

    def belongs_to_domain_named?(domain_name)
      !user_domains.where(name: domain_name).empty?
    end

    def is_super_user?
      has_role? :r_super_user
    end

    def is_developer?
      has_role? :r_developer
    end

    # rubocop:disable Metrics/AbcSize
    # rubocop:disable Naming/PredicateName

    # Determine if user has a role from the current_domain or from the 'default' domain
    # This could be a direct assignment or a role assigned to a group that the 'user' is associated to
    def has_role?(role)
      available_roles = []

      default_domain = Ugrp::Domain.default_domain

      # Directly assigned roles from 'default' domain
      available_roles.concat(roles.where(domain: default_domain))

      # Directly assigned roles from current domain
      available_roles.concat(roles.where(domain: current_domain || nil))

      # Group-based roles from assigned groups in the 'default' domain
      groups.where(domain: default_domain).each do |group|
        available_roles.concat(group.roles.where(domain: default_domain))
      end

      # Group-based roles from assigned groups in the current domain
      groups.where(domain: current_domain || nil).each do |group|
        available_roles.concat(group.roles.where(domain: current_domain || nil))
      end

      has_role = false
      available_roles.each do |available_role|
        has_role ||= (available_role.name == role.to_s)
      end

      logger.debug "has_role?(#{role}) = #{has_role}"
      has_role
    end

    # rubocop:enable Metrics/AbcSize
    # rubocop:enable Metrics/MethodLength
    # rubocop:enable Naming/PredicateName

    def ability
      @ability ||= Ugrp::Ability.new(self)
    end
    delegate :can?, :cannot?, to: :ability

    def ensure_authentication_token
      self.authentication_token = generate_authentication_token if authentication_token.blank?
    end

    #----------------------------------------------------------------------------

    def self.find_for_database_authentication(warden_conditions)
      conditions = warden_conditions.dup
      login_attribute = conditions.delete(:login)

      # Test for special (developer / super_user) accounts
      special_account = where(conditions)
                        .where(['lower(username) = :value OR lower(email) = :value', { value: login_attribute.strip.downcase }])
                        .first

      if special_account.present? && (special_account.is_developer? || special_account.is_super_user?)
        special_account
      else
        where(conditions)
          .where(['lower(username) = :value OR lower(email) = :value', { value: login_attribute.strip.downcase }])
          .joins(:user_domains).where(ugrp_user_domains: { domain_id: [Ugrp::DomainHelper.get_domain_from_request.id] })
          .first
      end
    end

    def self.find_for_cas_authentication(access_token, _signed_in_resource = nil)
      data = access_token.extra # .raw_info
      logger.debug "DEBUG -> DATA #{data}"
      user = User.where(username: data.user).first
      user ||= User.create!(email: "#{data.user}@cas-user.com",
                            username: data.user, name: data.user,
                            password: Devise.friendly_token[0, 20])

      user
    end

    def self.find_for_oauth_authentication(auth, _signed_in_resource = nil)
      user = User.where(provider: auth.provider, uid: auth.uid).first

      user ||= User.create(
        name => auth.extra.raw_info.name,
        provider => auth.provider,
        uid => auth.uid,
        email => auth.info.email,
        password => Devise.friendly_token[0, 20]
      )

      user
    end

    def self.find_for_oauth2_authentication(access_token, _signed_in_resource = nil)
      data = access_token.info
      user = User.where(email: data['email']).first

      user ||= User.create(
        name => data['name'],
        email => data['email'],
        password => Devise.friendly_token[0, 20]
      )

      user
    end

    #----------------------------------------------------------------------------

    # Attempt to find a user by it's email. If a record is found, send new
    # password instructions to it. If not user is found, returns a new user
    # with an email not found error.
    def self.send_reset_password_instructions(attributes = {})
      recoverable = find_recoverable_or_initialize_with_errors(reset_password_keys, attributes, :not_found)
      recoverable.send_reset_password_instructions if recoverable.persisted?
      recoverable
    end

    def self.find_recoverable_or_initialize_with_errors(required_attributes, attributes, error = :invalid)
      (case_insensitive_keys || []).each { |k| attributes[k].try(:downcase!) }

      attributes = attributes.slice(*required_attributes)
      attributes.delete_if { |_key, value| value.blank? }

      if attributes.size == required_attributes.size
        if attributes.key?(:login)
          login_attribute = attributes[:login]
          record = find_record(login_attribute)
        else
          record = where(attributes).first
        end
      end

      unless record
        record = new

        required_attributes.each do |key|
          value = attributes[key]
          record.send(:"#{key}=", value)
          record.errors.add(key, value.present? ? error : :blank)
        end
      end
      record
    end

    def self.find_record(login)
      where(['username = :value OR email = :value', { value: login }]).first
    end

    #----------------------------------------------------------------------------

    private

    def generate_authentication_token
      loop do
        token = Devise.friendly_token
        break token unless User.where(authentication_token: token).first
      end
    end
  end
end

# rubocop:enable Metrics/ClassLength
