require 'rails'
require 'devise'

module Ugrp
  class Engine < ::Rails::Engine
    isolate_namespace Ugrp

    initializer 'ugrp.application_controller' do |app|
      # Inject UGRP hooks/helpers into the main ApplicationController
      ActiveSupport.on_load(:action_controller) do
        extend  Ugrp::Controllers::ApplicationControllerExtensions::ClassMethods
        include Ugrp::Controllers::ApplicationControllerExtensions::InstanceMethods
        extend  Ugrp::Helpers::ApplicationHelperExtensions::ClassMethods
        include Ugrp::Helpers::ApplicationHelperExtensions::InstanceMethods

        if Rails.version.to_f >= 5.1
          before_action :update_ugrp_domain
          before_action :update_ugrp_user
        else
          before_filter :update_ugrp_domain
          before_filter :update_ugrp_user
        end

        helper_method :current_ugrp_domain
        helper_method :current_ugrp_user

        helper_method :login_path
        helper_method :logout_path
        helper_method :register_path
        helper_method :edit_user_registration_path
      end
    end

    # Make the asset pipeline from the following gem(s)/engine(s) available to the application
    require 'devise' # IMPORTANT: This line is location specific
    require 'recaptcha'
    require 'recaptcha/rails'

    config.generators do |g|
      g.test_framework :rspec, fixture_replacement: :factory_bot, views: false, helper: true
      g.view_specs false
      g.helper_specs true
      g.fixture_replacement :factory_bot, dir: 'spec/factories'
    end

    # initialize 'Ugrp.load_app_instance_data' do |app|
    #   Ugrp.setup do |config|
    #     config.app_root = main_app.root
    #   end
    # end

    # config.autoload_paths << File.expand_path("../validators", __FILE__)

    # monkey patch via: http://pivotallabs.com/leave-your-migrations-in-your-rails-engines/
    initializer "ugrp.migrations" do |app|
      unless app.root.to_s.match root.to_s
        config.paths['db/migrate'].expanded.each { |expanded_path| app.config.paths['db/migrate'] << expanded_path }
      end
    end

    initializer "ugrp.factories", after: "factory_bot.set_factory_paths" do
      if Rails.env.test?
        puts "Loading factories from #{self.class.name} engine"
        FactoryBot.definition_file_paths << File.expand_path('../../spec/factories', __dir__) if defined?(FactoryBot)
      end
    end

    # Allow translations from config/locales/*.rb,yml to be auto loaded.
    config.i18n.load_path += Dir[File.join(Ugrp::Engine.root, 'config', 'locales', '**', '*.{rb,yml}').to_s]

    # Execute local initializers for any in-house gems
    initializer 'ugrp.load_config_initializers', after: :load_config_initializers do |app|
      Rails.logger.info "Ugrp::Engine : Loading initializers"
      require File.join(File.dirname(__FILE__), '..', '..', 'config', 'in_house_gems')
      in_house_engines = UGRP_IN_HOUSE_ENGINES
      in_house_engines.each do |in_house_engine|
        initializer = File.expand_path(
          File.join(File.dirname(__FILE__), '..', '..', 'config', 'initializers', "#{in_house_engine[:name]}.rb")
        )
        Rails.logger.info "Ugrp::Engine : Loading '#{initializer}'"
        require initializer if File.exist? initializer
      end
    end
  end
end
