require 'active_record/fixtures'
require 'csv'

module Ugrp
  class SeedOperations
    ROOT_DIR = Ugrp::Engine.root

    def self.load_seed
      puts "Loading seed data from #{ROOT_DIR}"

      puts "ENVIRONMENT: #{Rails.env}"

      puts "Loading common seeds"

      import_data "seed/once/common"
      import_data "seed/always/common", :always

      load_acl_mappings("#{Rails.root.join('db/seed/once/common/ugrp')}")
      load_acl_mappings("#{Rails.root.join('db/seed/always/common/ugrp')}")

      puts "Loading #{Rails.env} specific seeds"

      import_data "seed/once/#{Rails.env}"
      import_data "seed/always/#{Rails.env}", :always

      load_acl_mappings("#{Rails.root.join("db/seed/once/#{Rails.env}/ugrp")}")
      load_acl_mappings("#{Rails.root.join("db/seed/always/#{Rails.env}/ugrp")}")
    end

    def self.import_data(dir, always = false)
      # load_fixtures(dir, always)
      load_csv_fixtures(dir, always)
      # load_acl_mappings(dir, always)
      load_complex_seeds(dir, always)
    end

    def self.load_fixtures(dir, always = false)
      Dir.glob(File.join(ROOT_DIR, 'db', dir, '*.yml')).sort.each do |fixture_file|
        table_name = File.basename(fixture_file, '.*')

        if table_empty?(table_name) || always || dir.include?("#{Rails.env}")
          print "Loading '#{table_name}' table"
          STDOUT.flush
          truncate_table(table_name)
          ActiveRecord::Fixtures.create_fixtures(File.join('db/', dir), table_name)
          puts " - done"
        else
          puts "Skipped '#{table_name}' table"
        end
      end
    end

    def self.load_csv_fixtures(dir, always = false)
      Dir.glob(File.join(ROOT_DIR, 'db', dir, '*.{csv}')).sort.each do |fixture_file|
        # reconstruct class and table names
        parts = File.basename(fixture_file, '.*').gsub(/(\d+)_/, '').split('-')
        parts.map! { |item| item.camelize }
        class_name = parts.join('::').classify
        table_name = File.basename(fixture_file, '.*').gsub(/(\d+)_/, '').tr("-", '_')

        if table_empty?(table_name) || always || dir.include?("#{Rails.env}")
          print "Loading '#{table_name}' table (#{class_name})"
          STDOUT.flush
          truncate_table(table_name)
          load_csv_file(fixture_file, class_name)
          puts " - done"
        else
          puts "Skipped '#{table_name}' table"
        end
      end
    end

    def self.load_complex_seeds(dir, always = false)
      Dir.glob(File.join(ROOT_DIR, 'db', dir, '*.{rb}')).sort.each do |seed_file|
        table_name = File.basename(seed_file, '.*').gsub(/(\d+)_/, '')

        if table_empty?(table_name) || always || dir.include?("#{Rails.env}")
          puts "Running '#{seed_file}' (#{table_name})"
          require seed_file
        else
          puts "Skipped '#{table_name}' table"
        end
      end
    end

    # Populate acl_mapping definitions
    def self.load_acl_mappings(dir, _always = false)
      Dir.glob(File.join(dir, '*.{csv}')).sort.each do |seed_file|
        table_name = File.basename(seed_file, '.*').gsub(/(.*)-.*/, '\1')
        domain_name = File.basename(seed_file, '.*').gsub(/.*-(.*)/, '\1')
        puts "Parsing '#{seed_file}' (#{table_name}/#{domain_name})"

        domain = Ugrp::Domain.find_by(url: (domain_name == 'default' ? nil : domain_name))

        line_no = -1
        # CSV.foreach(seed_file, headers: true, skip_blanks: true) do |row|
        # CSV.foreach(seed_file, headers: true) do |row|
        if domain
          CSV.foreach(seed_file) do |row|
            # CSV.open(seed_file, 'r') do |row|
            # CSV.foreach(seed_file, {headers: true, skip_blanks: true}) do |row|
            # input_data = CSV.new(seed_file, {headers: true, skip_blanks: true})
            # input_data.open() do |row|

            line_no += 1

            next if line_no == 0 # Assume and ignore header row

            # break if line_no > 2

            group_or_role_name = row[6]
            if group_or_role_name.starts_with?('g_')
              group_or_role = Ugrp::Group.find_by(domain: domain, name: group_or_role_name)
              if group_or_role.nil?
                group_or_role = Ugrp::Group.create(domain: domain, name: group_or_role_name,
                                                   descriptive_name: group_or_role_name)
              end
            else
              group_or_role = Ugrp::Role.find_by(domain: domain, name: group_or_role_name)
              if group_or_role.nil?
                group_or_role = Ugrp::Role.create(domain: domain, name: group_or_role_name,
                                                  descriptive_name: group_or_role_name)
              end
            end

            record = Ugrp::AclMapping.new(
              object_name: row[0],
              field_name: row[1],
              can_create: row[2],
              can_read: row[3],
              can_update: row[4],
              can_delete: row[5],
              group_or_role: group_or_role
            )
            record.save!
            print "." if line_no % 25 == 0
            STDOUT.flush
          end
        end
        print "*\n"
        STDOUT.flush
      end
    end

    def self.load_csv_file(filename, class_name)
      klass = class_name
      line_no = 0
      keys = []

      CSV.foreach(filename) do |row|
        if line_no == 0
          # First row is a header
          row.each do |field|
            keys << field
          end
        else
          # data
          attributes = {}
          field_number = 0
          keys.each do |key|
            attributes[key] = row[field_number].presence
            field_number += 1
          end

          # attributes.each do |attribute|
          #  print "#{attribute} "
          # end
          # print "\n"

          record = klass.constantize.new(attributes)
          record.save!
        end

        line_no += 1

        # Give some feedback for long running imports
        # print "." if line_no % 25 == 0
      end

      # print "\n" if line_no > 25
    end

    def self.table_empty?(table_name)
      quoted = connection.quote_table_name(table_name)
      connection.select_value("SELECT COUNT(*) FROM #{quoted}").to_i.zero?
    end

    def self.truncate_table(table_name)
      quoted = connection.quote_table_name(table_name)
      connection.execute("DELETE FROM #{quoted}")
    end

    def self.connection
      ActiveRecord::Base.connection
    end
  end
end
